using System;
using System.Threading;
using NxCoreAPI;

class NxCoreThread {
	private String filename;
	private int threadPermission;
	private bool successfulExclude = false;
	public NxCoreThread(String tapePath, int perm) {
		this.filename = tapePath;
		this.threadPermission = perm;
	}
	public unsafe void run() {
		int returnValue = NxCore.ProcessTape(filename, null,(uint) NxCore.NxCF_EXCLUDE_CRC_CHECK, 0, this.OnNxCoreCallback);
		NxCore.processReturnValue(returnValue);
	}

	unsafe int OnNxCoreCallback(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg) {
		if (pNxCoreMsg->MessageType == NxCore.NxMSG_STATUS && !successfulExclude) {
			int[] optionPermissions = { 2, 20, 21, 22 };
			for (int i = 0; i < 4; i++) {
				int permission = optionPermissions[i];
				if (threadPermission != permission && NxCore.ExcludeOpraExch(NxCore.NxPERMID_REMOVE | permission) != 0)
					return NxCore.NxCALLBACKRETURN_CONTINUE;
			}
			successfulExclude = true;
		}
		
		if (pNxCoreMsg->MessageType == NxCore.NxMSG_TRADE) {
			NxCoreHeader* pHeader = &pNxCoreMsg->coreHeader;
			NxOptionHdr* pnxOptionHdr = pHeader->pnxOptionHdr;
			if (pnxOptionHdr != null) {
				String symbol = new String(&pHeader->pnxStringSymbol->String);
				NxTime* pTimestamp = &pHeader->nxExgTimestamp;
				NxDate* pExpireDate = &pnxOptionHdr->nxExpirationDate;
				double strikePrice = NxCore.PriceToDouble(pnxOptionHdr->strikePrice, 7);
				String contractType;
				if (pnxOptionHdr->PutCall == 1)
					contractType = "Put";
				else
					contractType = "Call";

				NxCoreTrade* pTrade = &pNxCoreMsg->coreData.Trade;
				double price = NxCore.PriceToDouble(pTrade->Price, pTrade->PriceType);
				uint size = pTrade->Size;

				Console.WriteLine("Trade for {0:s} {1:s} expiring {2:d}/{3:d2}/{4:d2} with strike ${5:f2} at {6:d2}:{7:d2}:{8:d2} for {9:d} shares at ${10:f2}",
					symbol, contractType, pExpireDate->Year, pExpireDate->Month, pExpireDate->Day, strikePrice,
					pTimestamp->Hour, pTimestamp->Minute, pTimestamp->Second, size, price);

			}
		}
		return NxCore.NxCALLBACKRETURN_CONTINUE;
	}
}

class optionSample {
	static void Main(string[] args) {
		if (args.Length < 1)
			return;

		Thread[] threads = {
			new Thread((new NxCoreThread(args[0],2)).run),
			new Thread((new NxCoreThread(args[0], 20)).run),
			new Thread((new NxCoreThread(args[0], 21)).run),
			new Thread((new NxCoreThread(args[0], 22)).run)
		};
		foreach (var thread in threads)
			thread.Start();
		foreach (var thread in threads)
			thread.Join();
	}
}