using System;
using NxCoreAPI;

class DetailedTradeSample {
	static unsafe int OnNxCoreCallback(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg) {
		if (pNxCoreMsg->MessageType == NxCore.NxMSG_TRADE) {
			NxCoreHeader* pHeader = &pNxCoreMsg->coreHeader;
			String symbol = new String(&pHeader->pnxStringSymbol->String);
			NxTime* pTimestamp = &pHeader->nxExgTimestamp;
			String exchange = new String(NxCore.GetDefinedString(NxCore.NxST_EXCHANGE, pHeader->ReportingExg)).Split('|')[0];

			NxCoreTrade* pTrade = &pNxCoreMsg->coreData.Trade;
			double price = NxCore.PriceToDouble(pTrade->Price, pTrade->PriceType);
			uint size = pTrade->Size;
			uint sequence = pTrade->ExgSequence;

			String conditions = "";
			for (int i = 0; i < 4; i++) {
				byte condition = pTrade->ExtTradeConditions[i];
				String conditionName = new String(NxCore.GetDefinedString(NxCore.NxST_TRADECONDITION, condition));
				conditions += conditionName + "|";
			}

			String eligibility = "";
			if (pTrade->ConditionFlags != 0) {
				eligibility += "(Doesn't set";
				if ((pTrade->ConditionFlags & NxCore.NxTCF_NOHIGH) != 0)
					eligibility += " high";
				if ((pTrade->ConditionFlags & NxCore.NxTCF_NOLOW) != 0)
					eligibility += " low";
				if ((pTrade->ConditionFlags & NxCore.NxTCF_NOLAST) != 0)
					eligibility += " last";
				eligibility += ")";
			}
			if ((pTrade->PriceFlags & NxCore.NxTPF_EXGCANCEL) != 0)
				Console.WriteLine("Cancel for {0:s} at sequence {1:d} on {2:s} {3:d} shares at ${4:f2}",
					symbol, sequence, exchange, size, price);
			else
				Console.WriteLine("Trade for {0:s} at {1:d2}:{2:d2}:{3:d2} "+
					"sequence {4:d} on {5:s} for {6:d} shares at ${7:f2} {8:s}{9:s}",
					symbol, pTimestamp->Hour, pTimestamp->Minute, pTimestamp->Second, sequence, exchange, size, price, conditions, eligibility);
	  	}
		return NxCore.NxCALLBACKRETURN_CONTINUE;
	}

	static unsafe void Main(string[] args) {
		if (args.Length < 1)
			return;
		int returnValue = NxCore.ProcessTape(args[0], null, 0, 0, DetailedTradeSample.OnNxCoreCallback);
		NxCore.processReturnValue(returnValue);
	}
}
